package com.hisschemoller.model 
{
	import com.hisschemoller.model.vo.ProgramVO;
	import com.hisschemoller.util.sound.SoundVO;
	
	import org.puremvc.as3.interfaces.IProxy;
	import org.puremvc.as3.patterns.proxy.Proxy;
	
	import flash.display.BitmapData;
	import flash.display.Shape;
	import flash.utils.ByteArray;
	import flash.utils.Timer;	
	/**
	 * (c) Copyright LBi Lost Boys 2009
	 * @author wouter.hisschemoller
	 */
	public class SoundProxy extends Proxy implements IProxy 
	{
		public static const NAME : String = 'KeyDataProxy';
		public static const SOUND_CHANGED : String = 'soundLoaded';
		public static const GRAPHIC_UPDATED : String = 'GraphicUpdated';
		private static const BYTES_PER_SAMPLE : int = 8;
		private var mProgramVO : ProgramVO;
		private var mUpdateDelayTimer : Timer = new Timer(50, 1);
		private var mDrawShape : Shape = new Shape();
		private var mMinSamplesPerPixel : Number = 1;
		private var mMaxSamplesPerPixel : Number = 1;
		private var mCurrentSamplesPerPixel : Number = 1;
		private var mPollingFrequency : int = 1;
		private var mNumPolls : int;
		private var mBytesToSkip : int;
		private var mDisplayWidth : int = 430;
		private var mDisplayHeight : int = 160;
		private var mDisplayMiddle : int;
		private var mFirstSampleInView : int;
		private var mNumSamplesInView : int;
		private var mNumSamplesTotal : int;
		private var mVerticalZoomLevel : Number;
		private var mFilledGraphicTreshold : int = 60;
		private var mDragOriginX : Number;
		private var mDragOriginY : Number;
		private var mFirstSampleInViewOriginal : int;
		private var mSamplesPerPixelOriginal : Number;

		public function SoundProxy()
		{
			super(NAME);
		}

		/**
		 * Called by the Model when the Proxy is registered
		 */
		override public function onRegister() : void
		{
			mProgramVO = new ProgramVO();
			mProgramVO.soundGraphic = new BitmapData(mDisplayWidth, mDisplayHeight, true, 0xFF0000);
			
			mDisplayMiddle = mDisplayHeight / 2;
			mVerticalZoomLevel = mDisplayMiddle;
		}

		/**
		 * A maximum of one hour of sound are extracted.
		 * @param inSoundVO : SoundVO object with the sound to display
		 */
		public function changeSound(inSoundVO : SoundVO) : void 
		{
			mProgramVO.soundData = inSoundVO;
			if(mProgramVO.soundData.sound && (mProgramVO.soundData.byteArray.length == 0)) mNumSamplesTotal = mProgramVO.soundData.sound.extract(mProgramVO.soundData.byteArray, (44100 * 60 * 60), 0);
			else mNumSamplesTotal = 0;
			
			mMaxSamplesPerPixel = mNumSamplesTotal / mDisplayWidth;
			mFirstSampleInView = 0;
			
			currentSamplesPerPixel = mMaxSamplesPerPixel;
			
			sendNotification(SoundProxy.SOUND_CHANGED);
		}

		/**
		 * Resets the state of the proxy.
		 */
		public function clear() : void
		{
			mMaxSamplesPerPixel = 1;
			mCurrentSamplesPerPixel = 1;
			mPollingFrequency = 1;
			mFirstSampleInView = 0;
			mNumSamplesInView = 0;
			mNumSamplesTotal = 0;
			mVerticalZoomLevel = mDisplayMiddle;
			mProgramVO.soundData.sound = null;
			
			updateGraphic();
		}

		/**
		 * @param inOriginX : mouseX at MOUSE_DOWN
		 * @param inOriginY : mouseY at MOUSE_DOWN
		 */
		public function initializeDrag(inOriginX : Number, inOriginY : Number) : void
		{
			mDragOriginX = inOriginX;
			mDragOriginY = inOriginY;
			mFirstSampleInViewOriginal = mFirstSampleInView;
			mSamplesPerPixelOriginal = mCurrentSamplesPerPixel;
		}

		/**
		 * TODO: Vertical drag to zoom doesn't combine nicely with horizontal drag to reposition.
		 * @param inDragNewX : mouseX at MOUSE_MOVE
		 * @param inDragNewY : mouseY at MOUSE_MOVE
		 */
		public function updateDrag(inDragNewX : Number, inDragNewY : Number) : void
		{
			// Recalculate the position by horizontal mouse drag
			var firstSample : int = mFirstSampleInViewOriginal + ((mDragOriginX - inDragNewX) * mCurrentSamplesPerPixel);
			
			setFirstSampleInView(firstSample, true);
		}

		/**
		 * 
		 */
		public function get minSamplesPerPixel() : Number
		{
			return mMinSamplesPerPixel;
		}

		/**
		 * 
		 */
		public function get maxSamplesPerPixel() : Number
		{
			return mMaxSamplesPerPixel;
		}

		/**
		 * 
		 */
		public function get currentSamplesPerPixel() : Number
		{
			return mCurrentSamplesPerPixel;
		}

		/**
		 * 
		 */
		public function set currentSamplesPerPixel(inSamplesPerPixel : Number) : void
		{
			setCurrentSamplesPerPixel(inSamplesPerPixel);
		}

		/**
		 * 
		 */
		public function get firstSampleInView() : int
		{
			return mFirstSampleInView;
		}

		/**
		 * 
		 */
		public function set firstSampleInView(inFirstSample : int) : void
		{
			setFirstSampleInView(inFirstSample);
		}

		/**
		 * 
		 */
		public function get numSamplesInView() : int
		{
			return mNumSamplesInView;
		}

		/**
		 * 
		 */
		public function get numSamplesTotal() : int
		{
			return mNumSamplesTotal;
		}

		/**
		 * 
		 */
		public function set verticalZoomLevel(inLevel : Number) : void
		{
			mVerticalZoomLevel = mDisplayMiddle * inLevel;
			
			updateGraphic();
		}

		/**
		 * Sets the zoom level of the display. Measured in how many samples one horizontal pixel of the display represents.
		 * @param inSamplesPerPixel : How many samples one horizontal pixel of the display represents.
		 * @param inCenterSamplePosition : The center or focus of the zoom. 0.5 is the horizontal center. 
		 */
		private function setCurrentSamplesPerPixel(inSamplesPerPixel : Number, inCenterSamplePosition : Number = 0.5) : void
		{
			if(inSamplesPerPixel == mCurrentSamplesPerPixel) return;
			
			// get the sample that is the center of the zoom
			var centerSample : int = mFirstSampleInView + (mNumSamplesInView * inCenterSamplePosition);
			
			mCurrentSamplesPerPixel = Math.max(mMinSamplesPerPixel, Math.min(inSamplesPerPixel, mMaxSamplesPerPixel));
			mNumSamplesInView = mCurrentSamplesPerPixel * mDisplayWidth;
			mFirstSampleInView = centerSample - (mNumSamplesInView * inCenterSamplePosition);
			
			// polling stuff
			mPollingFrequency = Math.ceil(mCurrentSamplesPerPixel / 20);
			mNumPolls = int(mCurrentSamplesPerPixel / mPollingFrequency);
			mBytesToSkip = (mPollingFrequency - 1) * SoundProxy.BYTES_PER_SAMPLE;
			
			if(mFirstSampleInView < 0) mFirstSampleInView = 0;
			if((mFirstSampleInView + mNumSamplesInView) > mNumSamplesTotal) mFirstSampleInView = mNumSamplesTotal - mNumSamplesInView;
			
			updateGraphic();
		}

		/**
		 * Repositions the displayed part of the wave by setting the first visible sample at the left edge of the display.
		 * @param inFirstSample : The first visible sample at the left edge of the display.
		 * @param inDoUpdate : True if the display should be updated after the setting was changed.
		 */
		public function setFirstSampleInView(inFirstSample : int, inDoUpdate : Boolean = true) : void
		{
			if(inFirstSample == mFirstSampleInView) return;
			
			mFirstSampleInView = inFirstSample;
			mFirstSampleInView = Math.max(mFirstSampleInView, 0);
			mFirstSampleInView = Math.min(mFirstSampleInView, mNumSamplesTotal - mNumSamplesInView);
			
			if(inDoUpdate) updateGraphic();
		}

		/**
		 * Redraws the display.
		 */
		private function updateGraphic() : void
		{
			// delay between updates
			if(mUpdateDelayTimer.running) return;
			mUpdateDelayTimer.start();
			
			mDrawShape.graphics.clear();
			mDrawShape.graphics.lineStyle(1, 0x333333, 1);
			mDrawShape.graphics.moveTo(0, mDisplayMiddle);
			
			mProgramVO.soundData.byteArray.position = mFirstSampleInView * SoundProxy.BYTES_PER_SAMPLE;
			
			var samplePosition : int = mFirstSampleInView;
			
			// loop through the pixels of a waveform image's width
			for(var i : int = 0;i < mDisplayWidth; ++i)
			{
				var positiveWaveHeight : Number = 0.0;
				var negativeWaveHeight : Number = 0.0;
				
				// get minimum and maximum of all samples within pixel
				for(var j : int = 0;j < mNumPolls; ++j)
				{
					// j is increased by mPollingFrequency, so j might get greater than mCurrentSamplesPerPixel
					//if(j >= mCurrentSamplesPerPixel)
					//{
					// correct sample position and skip rest of the loop
					//	samplePosition += (j - mCurrentSamplesPerPixel);
					//	continue;
					//}
					if(samplePosition < mNumSamplesTotal)
					{
						// read the left channel
						var amplitude : Number = mProgramVO.soundData.byteArray.readFloat();
						// skip the right channel
						mProgramVO.soundData.byteArray.readFloat();
						// and jump to the next polling position
						mProgramVO.soundData.byteArray.position += mBytesToSkip;
					}
					else
					{
						amplitude = 0;
					}
					
					if(amplitude > 0) positiveWaveHeight = Math.max(positiveWaveHeight, amplitude);
					else negativeWaveHeight = Math.min(negativeWaveHeight, amplitude);
					
					samplePosition += mPollingFrequency;
				}
				
				if(mCurrentSamplesPerPixel > mFilledGraphicTreshold)
				{
					mDrawShape.graphics.moveTo(i, mDisplayMiddle + (negativeWaveHeight * mVerticalZoomLevel));
					mDrawShape.graphics.lineTo(i, mDisplayMiddle + (positiveWaveHeight * mVerticalZoomLevel));
				}
				else
				{
					if(positiveWaveHeight > (negativeWaveHeight * -1))
					{
						mDrawShape.graphics.lineTo(i, mDisplayMiddle + (positiveWaveHeight * mVerticalZoomLevel));
					}
					else
					{
						mDrawShape.graphics.lineTo(i, mDisplayMiddle + (negativeWaveHeight * mVerticalZoomLevel));
					}
				}
			}
			
			// clear and redraw the bitmapdata content
			mProgramVO.soundGraphic.fillRect(mProgramVO.soundGraphic.rect, 0);
			mProgramVO.soundGraphic.draw(mDrawShape);
			
			sendNotification(SoundProxy.GRAPHIC_UPDATED, mProgramVO.soundGraphic);
		}
	}
}
