package com.hisschemoller.util.sound 
{
	import yalog.Yalog;
	
	import flash.events.ErrorEvent;
	import flash.events.Event;
	import flash.events.EventDispatcher;
	import flash.events.IOErrorEvent;
	import flash.events.ProgressEvent;
	import flash.events.SecurityErrorEvent;
	import flash.media.Sound;
	import flash.net.URLRequest;	
	/**
	 * @author Wouter Hisschemöller
	 * (c) May 7, 2009
	 */
	public class SoundLoader extends EventDispatcher 
	{
		private static const THE_INSTANCE : SoundLoader = new SoundLoader();
		private var mWaitingStack : Vector.<SoundVO> = new Vector.<SoundVO>();
		private var mLoadingStack : Vector.<SoundVO> = new Vector.<SoundVO>();
		// The maximum number of simultaneous loading MP3 files.
		private var mLoaderCount : uint = 2;

		public static function getInstance() : SoundLoader
		{
			return THE_INSTANCE;
		}

		/**
		 * it's a singleton
		 */
		public function SoundLoader()
		{
			if(THE_INSTANCE) throw new Error("Singleton: use SoundLoader.getInstance()");
		}

		/**
		 * 
		 */
		public function loadSound(inSoundVO : SoundVO) : void
		{
			if((inSoundVO.url == null) || (inSoundVO.url.length == 0))
			{
				Yalog.error("loadSound: sound url doesn't exist.", toString());
				return;
			}
			
			mWaitingStack.push(inSoundVO);
			
			loadNext();
		}

		/**
		 * Stops loading of all sounds and clears the loading stack.
		 */
		public function stopLoadingAll() : void 
		{
			for(var i : uint = 0;i < mLoadingStack.length; ++i) 
			{
				mLoadingStack[i].sound.close();
			}
			
			mLoadingStack = new Vector.<SoundVO>();
		}

		/**
		 * Stops loading of sound in inSoundVO.
		 * @param inSoundVO: identifying SoundVO
		 */
		public function stopLoadingSound(inSoundVO : SoundVO) : void 
		{
			for(var i : uint = 0;i < mLoadingStack.length; ++i) 
			{
				if (mLoadingStack[i] == inSoundVO)
				{
					mLoadingStack[i].sound.close();
					mLoadingStack.splice(i, 1);
					return;
				}
			}
		}

		/**
		 * @return The total number of loaded bytes of all working Loader objects.
		 */
		public function getTotalBytesLoaded() : uint 
		{
			return getTotalCount("bytesLoaded");
		}

		/**
		 * @return The total number of bytesTotal ("the compressed bytes in the entire media file") of all working Loader objects.
		 */
		public function getTotalBytesCount() : uint 
		{
			return getTotalCount("bytesTotal");
		}

		/**
		 * @param inProperty: the property to calculate; either "bytesLoaded" or "bytesTotal"
		 * @return The sum of numbers of this property.
		 */
		private function getTotalCount(inProperty : String) : uint 
		{
			var count : uint = 0;
			
			for (var i : uint = 0;i < mLoadingStack.length; ++i) 
			{
				if(inProperty == "bytesLoaded") count += mLoadingStack[i].sound.bytesLoaded;
				else if(inProperty == "bytesTotal") count += mLoadingStack[i].sound.bytesTotal;
			}
			return count;
		}

		/**
		 * Load next sound if the waiting stack is not empty.
		 * @sends SoundLoaderEvent#ALL_LOADED
		 */
		private function loadNext() : void 
		{
			// quit if all loaders taken
			if(mLoadingStack.length == mLoaderCount) return;

			// quit if no waiting data
			if(mWaitingStack.length == 0) 
			{
				if(mLoadingStack.length == 0) dispatchEvent(new SoundLoaderEvent(SoundLoaderEvent.ALL_LOADED));
				return;
			}
			
			// get next object to load
			var soundVO : SoundVO = mWaitingStack.shift();
			
			// create listeners
			soundVO.sound.addEventListener(Event.COMPLETE, handleLoaderEvent);
			soundVO.sound.addEventListener(Event.OPEN, handleLoadStarted);
			soundVO.sound.addEventListener(ProgressEvent.PROGRESS, handleLoaderProgressEvent);
			soundVO.sound.addEventListener(IOErrorEvent.IO_ERROR, handleLoaderEvent);
			soundVO.sound.addEventListener(SecurityErrorEvent.SECURITY_ERROR, handleLoaderEvent);
			
			// store object in loading list
			mLoadingStack.push(soundVO);
			
			// load sound
			soundVO.sound.load(new URLRequest(soundVO.url));
		}

		/**
		 * @sends SoundLoaderEvent#START
		 */
		private function handleLoadStarted(inEvent : Event) : void 
		{
			var soundVO : SoundVO = getSoundVOBySound(inEvent.target as Sound, mLoadingStack);
			
			if(soundVO == null) 
			{
				Yalog.error("handleLoaderEvent: data for loader not found", toString());
				return;
			}

			soundVO.isLoading = true;
			var event : SoundLoaderEvent = new SoundLoaderEvent(SoundLoaderEvent.START, soundVO);
			dispatchEvent(event);
		}

		/**
		 * @sends SoundLoaderEvent#ERROR
		 * @sends SoundLoaderEvent#COMPLETE
		 */
		private function handleLoaderEvent(inEvent : Event) : void 
		{
			var soundVO : SoundVO = getSoundVOBySound(inEvent.target as Sound, mLoadingStack);
			
			if(soundVO == null) 
			{
				Yalog.error("handleLoaderEvent: data for loader not found", toString());
				return;
			}
			
			if(inEvent is ErrorEvent) 
			{
				var errorEvent : ErrorEvent = inEvent as ErrorEvent;
				var event : SoundLoaderEvent = new SoundLoaderEvent(SoundLoaderEvent.ERROR, soundVO);
				event.error = errorEvent.text;
			} 
			else 
			{
				soundVO.isLoaded = true;
				soundVO.isLoading = false;
				event = new SoundLoaderEvent(SoundLoaderEvent.COMPLETE, soundVO);
			}
			trace(event.toString());
			dispatchEvent(event);
			
			// remove data from stack
			mLoadingStack.splice(mLoadingStack.indexOf(soundVO), 1);
			
			// remove listeners from finished sound
			unregisterListeners(soundVO);
			
			// continue loading
			loadNext();
		}

		/**
		 * Handle ProgressEvent from Sound.
		 * @param e: ProgressEvent sent
		 * @sends AssetLoaderEvent#PROGRESS
		 */
		private function handleLoaderProgressEvent(inEvent : ProgressEvent) : void 
		{
			var soundVO : SoundVO = getSoundVOBySound(inEvent.target as Sound, mLoadingStack);
			
			if(soundVO == null) 
			{
				Yalog.error("handleLoaderEvent: data for loader not found", toString());
				return;
			}
			
			dispatchEvent(new SoundLoaderEvent(SoundLoaderEvent.PROGRESS, soundVO));
		}

		/**
		 * remove all listeners from a sound that finished loading
		 * @param inSoundVO : the SoundVO containing the finished sound
		 */
		private function unregisterListeners(inSoundVO : SoundVO) : void
		{
			inSoundVO.sound.removeEventListener(Event.COMPLETE, handleLoaderEvent);
			inSoundVO.sound.removeEventListener(Event.OPEN, handleLoadStarted);
			inSoundVO.sound.removeEventListener(ProgressEvent.PROGRESS, handleLoaderProgressEvent);
			inSoundVO.sound.removeEventListener(IOErrorEvent.IO_ERROR, handleLoaderEvent);
			inSoundVO.sound.removeEventListener(SecurityErrorEvent.SECURITY_ERROR, handleLoaderEvent);
		}

		/**
		 * Get sound data for specified sound
		 */
		private function getSoundVOBySound(inSound : Sound, inStack : Vector.<SoundVO>) : SoundVO 
		{
			for each (var soundVO : SoundVO in inStack)
			{
				if(soundVO.sound == inSound) return soundVO;
			}

			return null;
		}
	}
}
