package com.hisschemoller.view.components.base 
{
	import com.hisschemoller.view.components.base.Component;
	
	import flash.display.MovieClip;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.events.TimerEvent;
	import flash.geom.Rectangle;
	import flash.utils.Timer;	
	/**
	 * (c) Copyright LBi Lost Boys 2009
	 * @author wouter.hisschemoller
	 */
	public class ScrollBar extends Component 
	{
		public static const HORIZONTAL : String = "horizontal";
		public static const VERTICAL : String = "vertical";
		protected var mOrientation : String;
		protected var mTrack : MovieClip;
		protected var mThumb : MovieClip;
		protected var mPageSize : Number;
		protected var mMinScrollPosition : Number;
		protected var mMaxScrollPosition : Number;
		protected var mPageScrollSize : Number;
		protected var mScrollPosition : Number = 0;
		protected var mIsScrolling : Boolean = false;
		protected var mTrackScrollTimer : Timer;
		protected var mPageScrollDirection : int;

		public function ScrollBar(inAsset : MovieClip, inOrientation : String = ScrollBar.HORIZONTAL)
		{
			mOrientation = inOrientation;
			
			super(null, inAsset);
		}

		/**
		 * Convenience method to set the four main properties in one shot.
		 * @param inPageSize : Number which represents the size of one page.
		 * @param inMinScrollPosition : Number which represents the bottom of the scrolling range.
		 * @param inMaxScrollPosition : Number which represetns the top of the scrolling range.
		 * @param inPageScrollSize : Number which represents the increment to move when the scroll track is pressed.
		 * @param inScrollPosition : The current value of the slider.
		 */
		public function setScrollProperties(inPageSize : Number, inMinScrollPosition : Number, inMaxScrollPosition : Number, inPageScrollSize : Number = 0, inScrollPosition : Number = 0) : void
		{
			pageSize = inPageSize;
			minScrollPosition = inMinScrollPosition;
			maxScrollPosition = inMaxScrollPosition;
			pageScrollSize = inPageScrollSize;
			scrollPosition = inScrollPosition;
		}

		/**
		 * Sets / gets the size of one page.
		 */
		public function set pageSize(inPageSize : Number) : void
		{
			mPageSize = inPageSize;
			correctValue();
			invalidate();
		}

		public function get pageSize() : Number
		{
			return mPageSize;
		}

		/**
		 * Sets / gets the increment to move when the scroll track is pressed.
		 */
		public function set pageScrollSize(inPageScrollSize : Number) : void
		{
			mPageScrollSize = inPageScrollSize;
		}

		public function get pageScrollSize() : Number
		{
			return mPageScrollSize;
		}

		/**
		 * Sets / gets the bottom of the scrolling range.
		 */
		public function set minScrollPosition(inMinScrollPosition : Number) : void
		{
			mMinScrollPosition = inMinScrollPosition;
			correctValue();
			invalidate();
		}

		public function get minScrollPosition() : Number
		{
			return mMinScrollPosition;
		}

		/**
		 * Sets / gets the top of the scrolling range.
		 */
		public function set maxScrollPosition(inMaxScrollPosition : Number) : void
		{
			mMaxScrollPosition = inMaxScrollPosition;
			correctValue();
			invalidate();
		}

		public function get maxScrollPosition() : Number
		{
			return mMaxScrollPosition;
		}

		/**
		 * Sets / gets the current value of this slider.
		 */
		public function set scrollPosition(inPosition : Number) : void
		{
			if(mIsScrolling) return;
			
			mScrollPosition = inPosition;
			correctValue();
			invalidate();
		}

		public function get scrollPosition() : Number
		{
			return mScrollPosition;
		}

		/**
		 * Adjusts position of thumb when mScrollPosition, mMinScrollPosition, mMaxScrollPosition or mPageSize have changed.
		 * TODO: Should also be called when slider is resized.
		 */
		override public function draw() : void
		{
			if(mOrientation == ScrollBar.HORIZONTAL)
			{
				mThumb.width = Math.max(1, (mPageSize / ((mMaxScrollPosition + mPageSize) - mMinScrollPosition)) * mTrack.width);
				mThumb.x = (mScrollPosition - mMinScrollPosition) / (mMaxScrollPosition - mMinScrollPosition) * (mTrack.width - mThumb.width);
			}
			else
			{
				mThumb.height = Math.max(1, (mPageSize / ((mMaxScrollPosition + mPageSize) - mMinScrollPosition)) * mTrack.height);
				mThumb.y = (mScrollPosition - mMinScrollPosition) / (mMaxScrollPosition - mMinScrollPosition) * (mTrack.height - mThumb.height);
			}
		}
		
		/**
		 * 
		 */
		override protected function init() : void
		{
			super.init();
			
			mTrackScrollTimer = new Timer(200, 1);
			mTrackScrollTimer.addEventListener(TimerEvent.TIMER, handleTrackScrollTimer);
		}

		/**
		 * Adjusts value to be within minimum and maximum.
		 */
		protected function correctValue() : void
		{
			mScrollPosition = Math.min(mScrollPosition, mMaxScrollPosition);
			mScrollPosition = Math.max(mScrollPosition, mMinScrollPosition);
		}
		
		/**
		 * scroll one pageSize at a time (when track is clicked)
		 */
		protected function pageScroll() : void
		{
			var delta : Number = mPageScrollSize != 0 ? mPageScrollSize : mPageSize;
			var newPos : Number = mScrollPosition + (mPageScrollDirection * delta);
			
			if (newPos > mMaxScrollPosition) newPos = mMaxScrollPosition;
			else if (newPos < mMinScrollPosition) newPos = mMinScrollPosition;
			
			if(newPos != mScrollPosition)
			{
				scrollPosition = newPos;
				
				invalidate();
				dispatchEvent(new Event(Event.CHANGE));
			}
		}

		/**
		 * Handler called when user presses the mouse on the background of the scrollbar, causing the scrollBar to scroll a page at a time.
		 * @param inEvent : The MouseEvent passed by the system.
		 */
		protected function handleTrackMouseDown(inEvent : MouseEvent) : void
		{
			if(mOrientation == ScrollBar.HORIZONTAL)
			{
				if(mAsset.mouseX < mThumb.x) mPageScrollDirection = -1;
				else mPageScrollDirection = 1;
			}
			else
			{
				if(mAsset.mouseY < mThumb.y) mPageScrollDirection = -1;
				else mPageScrollDirection = 1;
			}
			
			mTrackScrollTimer.start();
			
			pageScroll();
		}

		/**
		 * Handler called when user releases the mouse on the background of the scrollbar.
		 * @param inEvent : The MouseEvent passed by the system.
		 */
		protected function handleTrackMouseUp(inEvent : MouseEvent) : void
		{
			mTrackScrollTimer.reset();
			mTrackScrollTimer.delay = 200;
			mTrackScrollTimer.repeatCount = 1;
		}

		/**
		 * Internal mouseDown handler. Starts dragging the handle.
		 * @param event : The MouseEvent passed by the system.
		 */
		protected function handleThumbMouseDown(inEvent : MouseEvent) : void
		{
			mIsScrolling = true;
			
			mTrack.stage.addEventListener(MouseEvent.MOUSE_UP, handleStageMouseUp);
			mTrack.stage.addEventListener(MouseEvent.MOUSE_MOVE, handleStageMouseMove);
			
			if(mOrientation == ScrollBar.HORIZONTAL)
			{
				mThumb.startDrag(false, new Rectangle(0, 0, mTrack.width - mThumb.width, 0));
			}
			else
			{
				mThumb.startDrag(false, new Rectangle(0, 0, 0, mTrack.height - mThumb.height));
			}
		}

		/**
		 * Internal mouseUp handler. Stops dragging the handle.
		 * @param inEvent : The MouseEvent passed by the system.
		 */
		protected function handleStageMouseUp(inEvent : MouseEvent) : void
		{
			mIsScrolling = false;
			
			mTrack.stage.removeEventListener(MouseEvent.MOUSE_UP, handleStageMouseUp);
			mTrack.stage.removeEventListener(MouseEvent.MOUSE_MOVE, handleStageMouseMove);
			
			mThumb.stopDrag();
		}

		/**
		 * Internal mouseMove handler for when the handle is being moved.
		 * @param inEvent : The MouseEvent passed by the system.
		 */
		protected function handleStageMouseMove(inEvent : MouseEvent) : void
		{
			var oldPosition : Number = mScrollPosition;
			
			if(mOrientation == ScrollBar.HORIZONTAL)
			{
				mScrollPosition = mThumb.x / (mTrack.width - mThumb.width) * (mMaxScrollPosition - mMinScrollPosition) + mMinScrollPosition;
			}
			else
			{
				mScrollPosition = mThumb.y / (mTrack.height - mThumb.height) * (mMaxScrollPosition - mMinScrollPosition) + mMinScrollPosition;
				// mScrollPosition = (mTrack.height - mThumb.height - mThumb.y) / (mTrack.height - mThumb.height) * (mMaxScrollPosition - mMinScrollPosition) + mMinScrollPosition;
			}
			
			if(mScrollPosition != oldPosition)
			{
				dispatchEvent(new Event(Event.CHANGE));
			}
		}

		/**
		 * Repeatedly called by mTrackScrollTimer while the mouse is pressed on the scrollbar track.
		 * @param inEvent : The MouseEvent passed by the system.
		 */
		protected function handleTrackScrollTimer(inEvent : TimerEvent) : void
		{
			if(mTrackScrollTimer.repeatCount == 1)
			{
				mTrackScrollTimer.delay = 100;
				mTrackScrollTimer.repeatCount = 0;
			}
			
			pageScroll();
		}
	}
}
