package com.hisschemoller.view.components.base 
{
	import com.hisschemoller.view.components.base.Component;

	import flash.display.MovieClip;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.geom.Rectangle;	
	/**
	 * (c) Copyright LBi Lost Boys 2009
	 * @author wouter.hisschemoller
	 */
	public class Slider extends Component 
	{
		public static const HORIZONTAL : String = "horizontal";
		public static const VERTICAL : String = "vertical";
		protected var mOrientation : String;
		protected var mTrack : MovieClip;
		protected var mThumb : MovieClip;
		private var mValue : Number = 0;
		private var mMinimum : Number = 0;
		private var mMaximum : Number = 100;
		private var mEnableTrackClick : Boolean = true;

		public function Slider(inAsset : MovieClip, inOrientation : String = Slider.HORIZONTAL)
		{
			mOrientation = inOrientation;
			
			super(null, inAsset);
		}

		/**
		 * Convenience method to set the three main properties in one shot.
		 * @param inMinimum The minimum value of the slider.
		 * @param inMaximum The maximum value of the slider.
		 * @param inValue The value of the slider.
		 */
		public function setSliderProperties(inMinimum : Number, inMaximum : Number, inValue : Number) : void
		{
			minimum = inMinimum;
			maximum = inMaximum;
			value = inValue;
		}

		/**
		 * Sets / gets whether or not a click on the background of the slider will move the handler to that position.
		 */
		public function set trackClickEnabled(inEnabled : Boolean) : void
		{
			mEnableTrackClick = inEnabled;
			invalidate();
		}

		public function get trackClickEnabled() : Boolean
		{
			return mEnableTrackClick;
		}

		/**
		 * Sets / gets the current value of this slider.
		 */
		public function set value(inValue : Number) : void
		{
			var oldValue : Number = mValue;
			
			mValue = inValue;
			correctValue();
			positionThumb();
			
			if(mValue != oldValue) dispatchEvent(new Event(Event.CHANGE));
		}

		public function get value() : Number
		{
			return mValue;
		}

		/**
		 * Gets / sets the maximum value of this slider.
		 */
		public function set maximum(inMaximum : Number) : void
		{
			mMaximum = inMaximum;
			correctValue();
			positionThumb();
		}

		public function get maximum() : Number
		{
			return mMaximum;
		}

		/**
		 * Gets / sets the minimum value of this slider.
		 */
		public function set minimum(inMinimum : Number) : void
		{
			mMinimum = inMinimum;
			correctValue();
			positionThumb();
		}

		public function get minimum() : Number
		{
			return mMinimum;
		}

		/**
		 * Adjusts value to be within minimum and maximum.
		 */
		protected function correctValue() : void
		{
			if(mMaximum > mMinimum)
			{
				mValue = Math.min(mValue, mMaximum);
				mValue = Math.max(mValue, mMinimum);
			}
			else
			{
				mValue = Math.max(mValue, mMaximum);
				mValue = Math.min(mValue, mMinimum);
			}
		}

		/**
		 * Adjusts position of thumb when value, maximum or minimum have changed.
		 * TODO: Should also be called when slider is resized.
		 */
		protected function positionThumb() : void
		{
			var range : Number;
			
			if(mOrientation == Slider.HORIZONTAL)
			{
				range = mTrack.width - mThumb.width;
				mThumb.x = (mValue - mMinimum) / (mMaximum - mMinimum) * range;
			}
			else
			{
				range = mTrack.height - mThumb.height;
				mThumb.y = mTrack.height - mThumb.height - (mValue - mMinimum) / (mMaximum - mMinimum) * range;
			}
		}

		/**
		 * Handler called when user clicks the background of the slider, causing the handle to move to that point. Only active if backClick is true.
		 * @param inEvent : The MouseEvent passed by the system.
		 */
		protected function handleTrackClick(inEvent : MouseEvent) : void
		{
			if(mOrientation == Slider.HORIZONTAL)
			{
				mThumb.x = mAsset.mouseX - (mThumb.width / 2);
				mThumb.x = Math.max(mThumb.x, 0);
				mThumb.x = Math.min(mThumb.x, mTrack.width - mThumb.width);
				mValue = mThumb.x / (mTrack.width - mThumb.width) * (mMaximum - mMinimum) + mMinimum;
			}
			else
			{
				mThumb.y = mAsset.mouseY - (mThumb.height / 2);
				mThumb.y = Math.max(mThumb.y, 0);
				mThumb.y = Math.min(mThumb.y, mTrack.height - mThumb.height);
				mValue = (mTrack.height - mThumb.height - mThumb.y) / (mTrack.height - mThumb.height) * (mMaximum - mMinimum) + mMinimum;
			}
			
			mThumb.dispatchEvent(new MouseEvent(MouseEvent.MOUSE_DOWN));
			
			dispatchEvent(new Event(Event.CHANGE));
		}

		/**
		 * Internal mouseDown handler. Starts dragging the handle.
		 * @param event : The MouseEvent passed by the system.
		 */
		protected function handleThumbMouseDown(inEvent : MouseEvent) : void
		{
			mTrack.stage.addEventListener(MouseEvent.MOUSE_UP, handleStageMouseUp);
			mTrack.stage.addEventListener(MouseEvent.MOUSE_MOVE, handleStageMouseMove);
			
			if(mOrientation == Slider.HORIZONTAL)
			{
				mThumb.startDrag(false, new Rectangle(0, 0, mTrack.width - mThumb.width, 0));
			}
			else
			{
				mThumb.startDrag(false, new Rectangle(0, 0, 0, mTrack.height - mThumb.height));
			}
		}

		/**
		 * Internal mouseUp handler. Stops dragging the handle.
		 * @param inEvent : The MouseEvent passed by the system.
		 */
		protected function handleStageMouseUp(inEvent : MouseEvent) : void
		{
			mTrack.stage.removeEventListener(MouseEvent.MOUSE_UP, handleStageMouseUp);
			mTrack.stage.removeEventListener(MouseEvent.MOUSE_MOVE, handleStageMouseMove);
			
			mThumb.stopDrag();
		}

		/**
		 * Internal mouseMove handler for when the handle is being moved.
		 * @param inEvent : The MouseEvent passed by the system.
		 */
		protected function handleStageMouseMove(inEvent : MouseEvent) : void
		{
			var oldValue : Number = mValue;
			
			if(mOrientation == Slider.HORIZONTAL)
			{
				mValue = mThumb.x / (mTrack.width - mThumb.width) * (mMaximum - mMinimum) + mMinimum;
			}
			else
			{
				mValue = (mTrack.height - mThumb.height - mThumb.y) / (mTrack.height - mTrack.width) * (mMaximum - mMinimum) + mMinimum;
			}
			
			if(mValue != oldValue) dispatchEvent(new Event(Event.CHANGE));
		}
	}
}
